#include <iostream>
#include <cmath>
#include <list> //seznam

using namespace std;

class ctyruhelnik
{
protected:
	double m_a;
	double m_b;
	double m_c;
	double m_d;
public:
	ctyruhelnik(double a, double b, double c, double d): m_a(a),
							     m_b(b),	
							     m_c(c),
							     m_d(d)
	{
		if ((a<=0)||(b<=0)||(c<=0)||(d<=0)) throw "Chyba: nekladna delka strany.";
	}
	
	virtual ~ ctyruhelnik(void){} 		//destruktor

	virtual double obsah (void)=0; 		//nebude existovat ctyruhelnik, ktery neni rovnobeznik ani lichobeznik (zjednoduseni) 

	double obvod (void)
	{
		return m_a+m_b+m_c+m_d;
	}
	
};

class rovnobeznik: public ctyruhelnik
{
public:
	rovnobeznik(double a, double b, double uhel): ctyruhelnik(a, b, a, b),
						      m_uhel(uhel)			//uhel sevreny stranou "a" a "b"
		
	{}

	virtual double obsah (void)
	{
		return m_a*m_b*sin(m_uhel);
	}
protected:
	double m_uhel;
};

class obdelnik: public rovnobeznik
{
public:
	obdelnik(double a, double b): rovnobeznik(a, b, 3.14/2) //uhel v radianech 90 stupnu = M_PI/2
	{}

	virtual double obsah (void)
	{
		return m_a*m_b;
	}
};

class ctverec: public obdelnik
{
public:
	ctverec(double a): obdelnik(a, a)
	{}

};

class lichobeznik: public ctyruhelnik
{
public:
	lichobeznik(double a, double b, double c, double d, double vyska): ctyruhelnik(a, b, c, d),   //rovnobezne strany jsou "a" a "c"
			   					           m_vyska(vyska)
	{
 		if (vyska<=0) throw "Chyba: nekladna vyska.";
	}

	virtual double obsah (void)
	{
		return m_vyska*(m_a+m_c)*0.5;
	}

protected:
	double m_vyska;
};

int main (void)
{
	double ObsahVsech=0;
	list <ctyruhelnik *> obrazce;
	
	obrazce.push_back(new rovnobeznik(4.0, 5.0, 3.14/4));	//uhel v radianech 90 stupnu = M_PI/4
	obrazce.push_back(new obdelnik(6.0, 7.0));
	obrazce.push_back(new ctverec(8.0));
	obrazce.push_back(new lichobeznik(4.0, 6.0, 8.0, 5.0, 4.5));
	for (_List_iterator<ctyruhelnik*> i=obrazce.begin();i!=obrazce.end();i++)	//pro vsechny objekty v seznamu obrazce
	//_List_iterator<ctyruhelnik*> i Je to iterator do seznamu ukazatelu na ctyruhelniky
	//v nasem pripade nemuzeme pouzit auto, protoze nemuzeme pouzit c++11, ktere nezna konstantu M_PI
	{
		cout<<(*i)->obsah()<<endl;
		ObsahVsech+=(*i)->obsah();
		delete *i; //uvolneni alokovane pameti (alokovano pomoci new)
	}
	cout<<"Obsah vsech ctyruhelniku je dohromady "<<ObsahVsech<<endl;
	return 0;
}



